(*******************************************************
 * Unit Lessons
 *
 * In dieser Unit werden Methoden und Klassen bereit-
 * gestellt, die zum Berarbeiten einer Lektionsdatei
 * ntig sind.
 *
 * Datum: 25.04.2000
 * Autor: Andreas Dangel
 * Version: 0.9.1b
 *
 * (c)2000 Andreas Dangel
 *******************************************************)
unit Lessons;

interface
uses classes;

{ Forward class declarations }
type TLessons    = class;
     TVocabulary = class;
     TVocabel    = class;

(*
 * Zeiger auf die jeweiligen Klassen, da oft mit
 * TList-Objekten gearbeitet wird.
 *)
     PLessons    = ^TLessons;
     PVocabulary = ^TVocabulary;
     PVocabel    = ^TVocabel;


(*
 * Klasse TLessons (TObject)
 *
 * Diese Klasse enthlt eine Liste, in der Instanzen
 * der Klasse TVocabulary gespeichert werden.
 * Bei Nachfrage wird FRawData mit den Daten
 * "gespeist", die dann genau so auf die Festplatte
 * gespeichert werden knnen.
 *)
TLessons = class(TObject)
  private
    (* speichert Instanzen von TVocabulary *)
    FList: TList;
    (* "rohe" Daten der gesamten Lektionen *)
    FRawData: TStringList;
    (* Methode, die die "rohen" Daten fr FRawData
     * erstellt.
     *)
    function _getRawData: TStringList;
  public
    constructor Create;
    destructor Destroy; override;

    (* Hier kann die komplette Liste ausgetauscht werden
     * Vorsicht: In der Liste drfen nur Zeiger auf
     * Instanzen von TVocabulary sein!
     *)
    procedure setList(list: TList);
    (* Mit dieser Methode kann man z.B. ganz einfach
     * eine Lektionsdatei einlesen und parsen lassen.
     *)
    procedure setRawData(data: TStringList);

    (* Methoden, die zur Bearbeitung von FList
     * bentigt werden (quivalente zu TList)
     *)
    procedure ListAdd(_p_vocabulary: PVocabulary);
    procedure ListClear;
    procedure ListDelete(index: Integer);
    procedure ListExchange(index1, index2: Integer);
    procedure ListInsert(index: Integer;
                         _p_vocabulary: PVocabulary);
    procedure ListRemove(_p_vocabulary: PVocabulary);

    (* Einfache Methoden um eine Lektionsdatei zu
     * ffnen und zu speichern.
     *)
    procedure LoadFromFile(filename: String);
    procedure SaveToFile(filename: String);

    (* Gibt die Liste zurck, um darauf zugreifen
     * zu knnen.
     *)
    function getList: TList;

    (* Gibt FRawData zurck *)
    function getRawData: TStringList;

    (* Erstellt eine Liste, die die Lektionsnamen
     * der Lektionen enthlt, die in FList eingetragen
     * sind.
     *)
    function getLessonsList: TStringList;

    (* Funktionen fr den Lexikon-Gebrauch einer
     * Lektionsdatei
     *)
    function SeekVocabel(vokabel: String): TStrings;
    function SeekBedeutung(bedeutung: String): TStrings;
  end;

(*
 * Klasse TVocabulary (TObject)
 *
 * Diese Klasse enthlt eine Liste, in der Instanzen der
 * Klasse TVocabel gespeichert werden. Auerdem besitzt
 * die Klasse TVocabulary noch ein Feld, das den Lektions-
 * namen beinhaltet.
 * Bei Nachfrage wird FRawData aktualisiert. FRawData
 * enthlt die Daten einer Lektion, die genau so auf die
 * Festplatte gespeichert werden knnen.
 *)
TVocabulary = class(TObject)
  private
    (* speichert Instanzen von TVocabel *)
    FList: TList;
    (* speichert den Lektionsnamen *)
    FName: String;
    (* die "rohen" Daten einer Lektion *)
    FRawData: TStringList;
    (* erstellt neue Instanz von TStringList und fllt
     * diese mit den "rohen" Daten.
     *)
    function _getRawData: TStringList;
  public
    constructor Create;
    destructor Destroy; override;

    (* setzt den Lektionsnamen *)
    procedure setName(text: String);
    (* Hier kann die komplette Liste ausgetauscht werden.
     * Vorsicht: Die Liste darf nur Zeiger auf Instanzen
     * von TVocabel enthalten!
     *)
    procedure setList(list: TList);

    (* Methoden, die zur Bearbeitung der Liste bentigt
     * werden (quivalent zu FList)
     *)
    procedure ListClear;
    procedure ListAdd(_p_vocabel: PVocabel);
    procedure ListDelete(index: Integer);
    procedure ListExchange(index1, index2: Integer);
    procedure ListInsert(index: Integer;
                         _p_vocabel: PVocabel);
    procedure ListRemove(_p_vocabel: PVocabel);

    (* Mit dieser Methode ist es mglich, Lektionen
     * zusammenzufgen, also aus 2 Lektionen eine
     * zu machen.
     *)
    procedure ListAddCopyVocabulary(
                      _p_vocabulary: PVocabulary);

    (* Aus Datei laden bzw. speichern *)
    procedure LoadFromFile(filename: String);
    procedure SaveToFile(filename: String);

    (* Mit dieser Methode kann man ganz einfach aus
     * einer Datei Daten fr eine komplette Lektion
     * einfgen.
     *)
    procedure setRawData(data: TStringList);

    (* gibt den Lektionsnamen zurck *)
    function getName: String;
    (* gibt FList zurck, um darauf zugreifen zu knnen *)
    function getList: TList;
    (* gibt FRawData zurck (die "rohen" Daten) *)
    function getRawData: TStringList;
    (* Erstellt eine Liste, die die Vokabeln aus FList
     * enthlt und gibt diese zurck.
     *)
    function getVokabelList: TStringList;
  end;

(*
 * Klasse TVocabel (TObject)
 *
 * Diese Klasse speichert die Daten, die fr genau eine
 * Vokabel bentigt werden, also Vokabel, Bedeutungen und
 * eine Bemerkung.
 * Bei Nachfrage wird FRawData aktualisiert. FRawData
 * enthlt die "rohen" Daten der Vokabel, was einer Zeile
 * in der Lektionsdatei entspricht.
 *)
TVocabel = class(TObject)
  private
    (* speichert die Vokabel *)
    FVokabel: String;
    (* speichert die Bedeutungen *)
    FBedeutungen: TStrings;
    (* speichert die Bemerkung *)
    FBemerkung: String;
    (* die "rohen" Daten der Vokabel *)
    FRawData: String;

    (* erstellt die "rohen" Daten *)
    function _getRawData: String;
  public
    constructor Create;
    destructor Destroy; override;

    (* verndert die Vokabel *)
    procedure setVokabel(text: String);
    (* setzt die BedeutungsListe neu *)
    procedure setBedeutungen(list: TStrings);
    (* Methoden, um die Bedeutungsliste zu bearbeiten *)
    procedure BedeutungenClear;
    procedure BedeutungenAdd(text: String);
    procedure BedeutungenDelete(index: Integer);
    procedure BedeutungenExchange(index1, index2: Integer);
    procedure BedeutungenInsert(index: Integer; text: String);
    procedure BedeutungenRemove(text: String);

    (* ndert die Bemerkung *)
    procedure setBemerkung(text: String);
    (* mit dieser Methode kann die Lektionsdatei zeilenweise
     * geparst werden.
     *)
    procedure setRawData(data: String);

    (* gibt die Vokabel zurck *)
    function getVokabel: String;
    (* gibt die Bedeutungen zurck (als TStrings!) *)
    function getBedeutungen: TStrings;
    (* gibt die Bemerkung zurck *)
    function getBemerkung: String;
    (* gibt FRawData zurck *)
    function getRawData: String;
  end;

(* dieses Zeichen wird als Trennzeichen in den Lektionsdateien
 * bentigt
 *)
const DELIMITER = '^';

implementation

(*
 * TLessons:
 * speichert eine Liste von TVocabulary
 *)
 constructor TLessons.Create;
 begin
   (* Initialisieren *)
   FList := TList.Create;
   FRawData := _getRawData;
   inherited Create;
 end;

 destructor TLessons.Destroy;
 var i: Integer;
     p_vocabulary: PVocabulary;
 begin
   (* zunchst alle Items der Liste zerstren *)
   for i := 0 to FList.Count-1 do
     begin
       p_vocabulary := FList[0];
       p_vocabulary^.Free;
       Dispose(p_vocabulary);
       FList.Delete(0);
     end;
   (* dann die Liste selber *)
   FList.Free;
   (* und FRawData: *)
   FRawData.Free;
   inherited Destroy;
 end;

(*
 * Diese Methode bestimmt RawData, das Format, das in den Lektionsdateien
 * vorliegt.
 *)
 function TLessons._getRawData: TStringList;
 var i: Integer;
     p_vocabulary: PVocabulary;
 begin
   (* ergebnis initialisieren *)
   Result := TStringList.Create;
   (* dann die Liste mit TVocabulary durchgehen
    * und dem Ergebnis hinzufgen
    *)
   for i := 0 to FList.Count-1 do
    begin
     p_vocabulary := FList[i];
     Result.AddStrings(p_vocabulary^.getRawData);
     Result.Add('');
    end;

   (* wenn letzte Zeile eine Leerzeile ist: lschen! *)
   (* die letzte Zeile lschen, wenn
    * - Result mehrere Zeilen besitzt, d.h. Result.Count > 0
    *   (da sonst -> Exception)
    * - die Zeile leer ist, also gleich ''
    *)
   if Result.Count > 0 then
        if Result[Result.Count-1] = '' then
            Result.Delete(Result.Count-1);
 end;

 procedure TLessons.setList(list: TList);
 begin
   (* FList zerstren *)
   FList.Free;
   (* Neuzuweisung *)
   FList := list;
 end;

 procedure TLessons.setRawData(data: TStringList);
 var i: Integer;
     temp: TStringList;
     p_vocabulary: PVocabulary;
 begin
   (* FList initialisieren, d.h. leeren *)
   FList.Clear;
   (* temp nimmt den Datenbestand von genau einer Lektion auf *)
   temp := TStringList.Create;

   (* jede Zeile durchgehen *)
   for i := 0 to data.Count-1 do
     begin
       (* wenn data[i] keine Leerzeile ist (das wrde eine neue Lektion
        * bedeuten)
        *)
       if data[i] <> '' then temp.Add(data[i])
         else (* die Zeile war leer *)
         begin
           (* neue "dynamische" Variable erstellen, initialisieren *)
           New(p_vocabulary);
           p_vocabulary^ := TVocabulary.Create;
           (* und mit Werten "fttern" *)
           p_vocabulary^.setRawData(temp);
           (* die Adresse der dynamischen Variablen (also der Lektion)
            * der Liste hinzufgen
            *)
           FList.Add(p_vocabulary);
            (* temp fr die nchste Lektion bereit machen *)
           temp.Clear;
         end;
     end;

   (* in Temp befindet sich jetzt noch die letzte
    * Lektion in der Datei
    * Bedingung: Die Lektionsdatei hat nicht mit einer Leerzeile
    * aufgehrt (dann wre der else-teil oben in Kraft getreten!)
    * Zur Sicherheit aber trotzdem:
    * Prfen, ob temp.Count > 0 ist, dann ist auch eine Lektion enthalten...
    *)
    if temp.Count > 0 then
      begin
       (* wie oben: dynamische Variable... *)
       New(p_vocabulary);
       p_vocabulary^ := TVocabulary.Create;
       p_vocabulary^.setRawData(temp);
       FList.Add(p_vocabulary);
      end;

   (* Auf jeden Fall: aufrumen *)
   temp.Free;
 end;

 procedure TLessons.ListAdd(_p_vocabulary: PVocabulary);
 begin
   (* fgt den Zeiger der Liste hinzu *)
   FList.Add(_p_vocabulary);
 end;

 procedure TLessons.ListClear;
 begin
   (* Leert die Liste *)
   FList.Free;
   FList := TList.Create;
 end;

 procedure TLessons.ListDelete(index: Integer);
 var p_vocabulary: PVocabulary;
 begin
   (* Zerstrt p_vocabulary und gibt den Speicherplatz
    * wieder frei.
    *)
   p_vocabulary := FList[index];
   p_vocabulary^.Free;
   Dispose(p_vocabulary);

   (* anschlieen aus der Liste Lschen! *)
   FList.Delete(index);
 end;

 procedure TLessons.ListExchange(index1, index2: Integer);
 begin
   (* tauscht index1 mit index2 *)
   FList.Exchange(index1, index2);
 end;

 procedure TLessons.ListInsert(index: Integer; _p_vocabulary: PVocabulary);
 begin
   (* Fgt an der Stelle index _p_vocabulary ein *)
   FList.Insert(index, _p_vocabulary);
 end;

 procedure TLessons.ListRemove(_p_vocabulary: PVocabulary);
 begin
   (* Entfernt _p_vocabulary aus der Liste, falls
    * vorhanden
    *)
   FList.Remove(_p_vocabulary);
   (* zerstrt das Objekt und gibt den Speicher frei *)
   _p_vocabulary^.Free;
   Dispose(_p_vocabulary);
 end;

 procedure TLessons.LoadFromFile(filename: String);
 var temp: TStringList;
 begin
  (* neue StringListe erstellen *)
  temp := TStringList.Create;
  (* und Daten aus filename laden *)
  temp.LoadFromFile(filename);
  (* parsen lassen *)
  setRawData(temp);
  (* StringListe wieder freigeben *)
  temp.Free;
 end;

 procedure TLessons.SaveToFile(filename: String);
 var temp: TStringList;
 begin
   (* temp mit FRawData fllen *)
   temp := getRawData;
   (* und in filename speichern *)
   temp.SaveToFile(filename);
   (* temp wird nicht mehr gebracht *)
   temp.Free;
 end;

 function TLessons.getList: TList;
 begin
   Result := FList;
 end;

 function TLessons.getRawData: TStringList;
 begin
   (* ganz wichtig: FRawData neu bestimmen! *)
   FRawData := _getRawData;
   (* und zurckgeben *)
   Result := FRawData;
 end;

 function TLessons.getLessonsList: TStringList;
 var i: Integer;
     p_vocabulary: PVocabulary;
 begin
   Result := TStringList.Create;
   (* Die Liste durchgehen *)
   for i := 0 to FList.Count-1 do
     begin
       p_vocabulary := FList[i];
       (* und jeweils den Namen der Lektion hinzufgen *)
       Result.Add(p_vocabulary^.getName);
     end;
 end;

 function TLessons.SeekVocabel(vokabel: String): TStrings;
 var i,j,k: Integer;
     p_vocabulary: PVocabulary;
     p_vocabel: PVocabel;
     s: String;
 begin
  Result := TStringList.Create;
  (* Alle Lektionen und Vokabeln durchgehen,
   * und mit pos() prfen, ob suchstring enthalten
   * ist.
   *)
  for i := 0 to FList.Count-1 do
   begin
     p_vocabulary := FList[i];
     for j := 0 to p_vocabulary^.getList.Count-1 do
       begin
         p_vocabel := p_vocabulary^.getList[j];
         (* mit pos() prfen: *)
         if pos(vokabel,p_vocabel^.getVokabel)>0 then
           begin
             (* string enthalten! *)
             s := p_vocabulary^.getName;
             s := s+': '+p_vocabel^.getVokabel+' - ';
             for k := 0 to p_vocabel^.getBedeutungen.Count-1 do
              s := s + p_vocabel^.getBedeutungen[k] + ' | ';
             (* letzte zeichen sind ' | '. Diese noch entfernen! *)
             s := copy(s,0,Length(s)-3);
             (* jetzt noch string dem ergebnis hinzufgen *)
             Result.Add(s);
           end;
       end;
   end;
   if Result.Count = 0 then Result.Add('Nichts gefunden!');
 end;

 function TLessons.SeekBedeutung(bedeutung: String): TStrings;
 var i,j,k,l: Integer;
     p_vocabulary: PVocabulary;
     p_vocabel: PVocabel;
     s: String;
 begin
  Result := TStringList.Create;
  (* Alle Lektionen und Vokabeln durchgehen,
   * und mit pos() prfen, ob suchstring enthalten
   * ist.
   *)
  for i := 0 to FList.Count - 1 do
    begin
     p_vocabulary := FList[i];
     for j := 0 to p_vocabulary^.getList.Count - 1 do
       begin
         p_vocabel := p_vocabulary^.getList[j];
         for k := 0 to p_vocabel^.getBedeutungen.Count - 1 do
           begin
             (* jetzt mit pos() prfen *)
             if pos(bedeutung,p_vocabel^.getBedeutungen[k]) > 0 then
               begin
                (* string enthalten! *)
                s := p_vocabulary^.getName;
                s := s+': '+p_vocabel^.getVokabel+' - ';
                for l := 0 to p_vocabel^.getBedeutungen.Count-1 do
                s := s + p_vocabel^.getBedeutungen[l] + ' | ';
                (* letzte zeichen sind ' | '. Diese noch entfernen! *)
                s := copy(s,0,Length(s)-3);
                (* jetzt noch string dem ergebnis hinzufgen *)
                Result.Add(s);
                (* jetzt die for-schleife, die die bedeutungen
                 * berprft, beenden:
                 *)
                Break;
               end;
           end;
       end;
    end;
   (* wenn nichts gefunden wurde, d.h.
    * wenn die Zeilenanzahl im Ergebnis = 0 ist
    *)
   if Result.Count = 0 then Result.Add('Nichts gefunden!');
 end;

(*
 * TVocabulary:
 * speichert eine Liste von TVocabel
 * und den dazugehrigen Lektionsnamen
 *)
 constructor TVocabulary.Create;
 begin
   (* alles initialisieren *)
   FList := TList.Create;
   FName := '';
   FRawData := _getRawData;
   inherited Create;
 end;

 destructor TVocabulary.Destroy;
 var i: Integer;
     p_vocabel: PVocabel;
 begin
   (* alle Items der Liste zersren *)
   for i := 0 to FList.Count-1 do
     begin
       p_vocabel := FList[0];
       p_vocabel^.Free;
       Dispose(p_vocabel);
       FList.Delete(0);
     end;
   (* schlielich die Listen-Objekte selbst *)
   FList.Free;
   FRawData.Free;
   inherited Destroy;
 end;

 function TVocabulary._getRawData: TStringList;
 var i: Integer;
     p_vocabel: PVocabel;
 begin
   Result := TStringList.Create;
   (* in der 1. Zeile steht der Name der Lektion *)
   Result.Add(FName);
   (* Liste durchgehen *)
   for i := 0 to FList.Count-1 do
     begin
       p_vocabel := FList[i];
       (* und zeile hinzufgen *)
       Result.Add(p_vocabel^.getRawData);
     end;
 end;

 procedure TVocabulary.setName(text: String);
 begin
   FName := text;
 end;

 procedure TVocabulary.setList(list: TList);
 begin
   FList.Free;
   FList := list;
 end;

 procedure TVocabulary.ListClear;
 begin
   FList.Free;
   FList := FList.Create;
 end;

 procedure TVocabulary.ListAdd(_p_vocabel: PVocabel);
 begin
   FList.Add(_p_vocabel);
 end;

 procedure TVocabulary.ListDelete(index: Integer);
 var p_vocabel: PVocabel;
 begin
   (* zuerst die Instanz zerstren *)
   p_vocabel := FList[index];
   p_vocabel^.Free;
   (* speicher freigeben *)
   Dispose(p_vocabel);
   (* aus der Liste lschen *)
   FList.Delete(index);
 end;

 procedure TVocabulary.ListExchange(index1, index2: Integer);
 begin
   FList.Exchange(index1, index2);
 end;

 procedure TVocabulary.ListInsert(index: Integer; _p_vocabel: PVocabel);
 begin
   FList.Insert(index, _p_vocabel);
 end;

 procedure TVocabulary.ListRemove(_p_vocabel: PVocabel);
 begin
   FList.Remove(_p_vocabel);
   (* da _p_vocabel ein Zeiger auf die zu lschende
    * Vokabel ist, kann dieser Zeiger einfach
    * dereferenziert werden.
    *)
   _p_vocabel^.Free;
   Dispose(_p_vocabel);
 end;

 procedure TVocabulary.ListAddCopyVocabulary(_p_vocabulary: PVocabulary);
 var _p_vocabel : PVocabel;
     p_vocabel  : PVocabel;
     i          : Integer;
 begin
  (* die Liste in _p_vocbulary durchgehen *)
   for i := 0 to _p_vocabulary^.getList.Count-1 do
     begin
       (* zeiger holen *)
       _p_vocabel := _p_vocabulary^.getList[i];
       (* neuen Speicher bereitstellen *)
       New(p_vocabel);
       (* Speicher initialisieren
        * muss nicht sein!!!
        *)
//       p_vocabel^ := TVocabel.Create;
(*
       p_vocabel^.setVokabel(_p_vocabel^.getVokabel);
       p_vocabel^.setBedeutungen(_p_vocabel^.getBedeutungen);
       p_vocabel^.setBemerkung(_p_vocabel^.getBemerkung);
 *)
       (* neu zuordnen, nur values *)
       p_vocabel^ := _p_vocabel^;
       (* jetzt noch der Liste hinzufgen *)
       FList.Add(p_vocabel);
     end;
   (* nach den Vokabeln noch den Lektionsnamen:
    * 2 Flle: FName = '' --> dann kein Komma!
    *          FName <>'' --> dann Komma!
    *)
   if FName <> '' then FName := FName + ', ';
   FName := FName + _p_vocabulary^.getName;
 end;


 procedure TVocabulary.LoadFromFile(filename: String);
 var temp: TStringList;
 begin
   temp := TStringList.Create;
   (* aus datei laden *)
   temp.LoadFromFile(filename);
   (* und parsen lassen *)
   setRawData(temp);
   temp.Free;
 end;

 procedure TVocabulary.SaveToFile(filename: String);
 begin
   (* einfach speichern *)
   getRawData.SaveToFile(filename);
 end;


 procedure TVocabulary.setRawData(data: TStringList);
 var i: Integer;
     p_vocabel: PVocabel;
 begin
   FList.Free;
   FList := TList.Create;

   (* 1. Zeile enthlt den Lektionsnamen *)
   FName := data[0];

   (* weitere Zeilen die Vokabeln *)
   for i := 1 to data.Count-1 do
     begin
      New(p_vocabel);
      p_vocabel^ := TVocabel.Create;
      p_vocabel^.setRawData(data[i]);
      FList.Add(p_vocabel);
     end;
 end;

 function TVocabulary.getName: String;
 begin
   Result := FName;
 end;

 function TVocabulary.getList: TList;
 begin
   Result := FList;
 end;

 function TVocabulary.getRawData: TStringList;
 begin
   (* FRawData neu bestimmen *)
   FRawData := _getRawData;
   Result := FRawData;
 end;

 function TVocabulary.getVokabelList: TStringList;
 var i: Integer;
     p_vocabel: PVocabel;
 begin
   Result := TStringList.Create;
   (* Liste durchgehen *)
   for i := 0 to FList.Count-1 do
     begin
       p_vocabel := FList[i];
       (* und dem Ergebnis die Vokabeln hinzufgen *)
       Result.Add(p_vocabel^.getVokabel);
     end;
 end;


(*
 * TVocabel:
 * speichert alle Informationen eines einzigen
 * Vokabeleintrags.
 *)
 constructor TVocabel.Create;
 begin
   (* initialisieren *)
   FVokabel := '';
   FBedeutungen := TStringList.Create;
   FBemerkung := '';
   FRawData := _getRawData;
   inherited Create;
 end;

 destructor TVocabel.Destroy;
 begin
   FBedeutungen.Free;
   inherited Destroy;
 end;

 function TVocabel._getRawData: String;
 var i: Integer;
 begin
   (* erstellt eine einzige Zeile in einer Lektionsdatei *)
   (* die zeile hat folgende Bauweise:
    * vokabel^bedeutung1^bedeutung_n^bemerkung
    *
    * DELIMITER = '^'
    *)
   Result := FVokabel + DELIMITER;
   for i := 0 to FBedeutungen.Count-1 do
       Result := Result + FBedeutungen[i] + DELIMITER;
   Result := Result + FBemerkung;
 end;

 procedure TVocabel.setVokabel(text: String);
 begin
   FVokabel := text;
 end;

 procedure TVocabel.setBedeutungen(list: TStrings);
 var i: Integer;
 begin
   (* FBedeutungen := list;
    * dies wurde nicht verwendet, weil in
    * BedeutungenAdd() sichergestellt wird, dass
    * keine "leeren" Bedeutungen hinzugefgt werden.
    *)
   FBedeutungen.Clear;
   for i := 0 to list.Count - 1 do
      BedeutungenAdd(list[i]);
 end;

 procedure TVocabel.BedeutungenClear;
 begin
   FBedeutungen.Clear;
 end;

 procedure TVocabel.BedeutungenAdd(text: String);
 begin
   (* keine Nullstrings hinzufgen! *)
   if text <> '' then FBedeutungen.Add(text);
 end;

 procedure TVocabel.BedeutungenDelete(index: Integer);
 begin
   FBedeutungen.Delete(index);
 end;

 procedure TVocabel.BedeutungenExchange(index1, index2: Integer);
 begin
   FBedeutungen.Exchange(index1, index2);
 end;

 procedure TVocabel.BedeutungenInsert(index: Integer; text: String);
 begin
   FBedeutungen.Insert(index, text);
 end;

 procedure TVocabel.BedeutungenRemove(text: String);
 var index: Integer;
 begin
   (* zuerst den Index bestimmen *)
   index := FBedeutungen.IndexOf(text);
   (* wenn der text vorkam, dann lschen *)
   if index > -1 then FBedeutungen.Delete(index);
 end;

 procedure TVocabel.setBemerkung(text: String);
 begin
   FBemerkung := text;
 end;

 procedure TVocabel.setRawData(data: String);
 begin
   (* FVokabel: 1 bis 1.DELIMITER *)
   FVokabel := copy(data, 1, pos(DELIMITER, data)-1);

   (* String rollen *)
   data := copy(data, pos(DELIMITER, data)+1, length(data));

   while (pos(DELIMITER, data) > 0) do
   (* d.h. DELIMITER kommt vor,
    * es gibt noch weitere Bedeutungen
    *)
   begin
     BedeutungenAdd(copy(data, 1, pos(DELIMITER, data)-1));
     (* String rollen *)
     data := copy(data, pos(DELIMITER, data)+1,Length(data));
   end;

   (* der Rest in data ist die Bemerkung *)
   FBemerkung := data;

   (* das war's *)
 end;

 function TVocabel.getVokabel: String;
 begin
   Result := FVokabel;
 end;

 function TVocabel.getBedeutungen: TStrings;
 begin
   Result := FBedeutungen;
 end;

 function TVocabel.getBemerkung: String;
 begin
   Result := FBemerkung;
 end;

 function TVocabel.getRawData: String;
 begin
   (* FRawData neu bestimmen *)
   FRawData := _getRawData;
   Result := FRawData;
 end;

end.

